/*
 * Auslesen eines Panasonic-Encoders mit wackligen Rastpunkten auf Spur B
 * Implementierung nach Vorschlag von Artikel "Drehgeber" auf
 * www.mikrocontroller.net
 *
 * Created: 09.07.2013 19:43:55
 *  Author: Nicolas
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "encoder_panasonic.h"
#include "ioMapping.h"

// Dekodertabelle fr wackeligen Rastpunkt (halbe Aufloesung)
const int8_t table[16] PROGMEM = {0,0,-1,0,0,0,0,1,1,0,0,0,0,-1,0,0};    
 
// Dekodertabelle fr normale Drehgeber (volle Aufloesung)
//const int8_t table[16] PROGMEM = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0};    


void encoder_init() {
	ENC_A_DDR  &= ~(1<<ENC_A_P);
	ENC_B_DDR  &= ~(1<<ENC_B_P);
	ENC_A_PORT |=  (1<<ENC_A_P);
	ENC_B_PORT |=  (1<<ENC_B_P);
}


// Encoder-Wert updaten
// sowohl pollen fuer internen Statusupdate als auch Auslesen
// der internen Werte
// Encoder-Wert wird fuer immer aufsummiert und nie zurueckgesetzt
uint8_t encoder_get() {
	static uint8_t enc_last = 0x01;
	static uint8_t enc_this = 0x00;		
    
    enc_last = (enc_last << 2)  & 0x0F;
    if (ENC_A_PIN & (1<<ENC_A_P) ) enc_last |=2;
    if (ENC_B_PIN & (1<<ENC_B_P) ) enc_last |=1;
    enc_this += pgm_read_byte(&table[enc_last]);
    return -enc_this;
}
